//
// Created by william on 12/23/23.
//

#include "include/cpu.h"
#include "include/system.h"
#include "memory.h"
#include <unistd.h>
#include <assert.h>
#include "cpu.h"
#include "pattern_table.h"

System current_sys;

void system_init() {
    byte *registers_base_addr = mem_get_ptr(PPU_REGISTERS_BASE_ADDR);
    byte *oam_dma_register = mem_get_ptr(PPU_REGISTER_OAM_DMA_ADDR);

    cpu_init();
    ppu_init(registers_base_addr, oam_dma_register);

    current_sys.mapper = get_mapper(MAPPER_TYPE_SIMPLE);
    current_sys.cycle_count = 7;
}

void system_start() {
    address pc = mem_get_word(0xfffc);
    cpu_get_state()->program_counter = pc;
}

void system_next_frame() {
    for (int cpu_c = 0; cpu_c < CPU_CYCLE_PER_FRAME; cpu_c++) {
        cpu_cycle();

        for (int ppu_c = 0; ppu_c < PPU_CYCLE_PER_CPU_CYCLE; ppu_c++) {
            ppu_cycle();
        }
    }

    pt_debug();
}

void system_uninit() {
    ppu_uninit();
}

unsigned int system_get_cycles() {
    return current_sys.cycle_count;
}

void system_add_cycles(unsigned int cycles) {
    current_sys.cycle_count += cycles;
}

Mapper *system_get_mapper() {
    return &current_sys.mapper;
}